import factory

from idissend.core import IncomingFile, Study, Stream, Person, Stage
from pathlib import Path


class MockIncomingFile(IncomingFile):
    """An IncomingFile which does not check on disk but just returns a set age
    value"""

    def __init__(self, path: Path, age: float):
        super(MockIncomingFile, self).__init__(path)
        self.age = lambda: age


class MockIncomingFileFactory(factory.Factory):
    class Meta:
        model = MockIncomingFile

    path = factory.sequence(lambda n: Path(f"mock_path_{n}"))
    age = 10


class PersonFactory(factory.Factory):
    class Meta:
        model = Person

    name = factory.Faker("first_name")
    email = factory.LazyAttribute(lambda a: f"{a.name}@example.com")


class StreamFactory(factory.Factory):
    class Meta:
        model = Stream

    name = factory.sequence(lambda n: f"stream_{n}")
    output_folder = factory.sequence(lambda n: Path(f"output_folder_for_stream_{n}"))
    idis_profile_name = factory.sequence(lambda n: f"idis_profile_name{n}")
    pims_key = factory.sequence(lambda n: f"111{n}")
    contact = factory.SubFactory(PersonFactory)


class StageFactory(factory.Factory):
    class Meta:
        model = Stage

    name = factory.sequence(lambda n: Path(f"Mock stage {n}"))
    path = factory.sequence(lambda n: Path(f"mock_path_{n}"))
    streams = factory.List([factory.SubFactory(StreamFactory) for _ in range(3)])


class StudyFactory(factory.Factory):
    class Meta:
        model = Study

    name = factory.sequence(lambda n: f"study_{n}")
    stream = factory.SubFactory(StreamFactory)
    stage = factory.SubFactory(StageFactory)



from datetime import datetime
from pathlib import Path

import pytest

from idissend.orm import PendingAnonRecord
from idissend.persistence import get_db_sessionmaker, IDISSendRecords


@pytest.fixture
def a_sqlite_url(tmpdir):
    db_file = Path(tmpdir) / "test_db.sqlite"
    db_url = f'sqlite:///{db_file}'
    return db_url


def test_db(a_sqlite_url):
    """Just very basic object creation in db. Check that it works as expected"""

    session = get_db_sessionmaker(a_sqlite_url)()

    study_folder = Path("/tmp/a_path")
    job_id = 123
    server_name = "p01"
    last_check = datetime.now()
    last_status = 4

    assert not session.query(PendingAnonRecord).all()
    a_record = PendingAnonRecord(
        study_folder=study_folder, job_id=job_id, server_name=server_name
    )
    session.add(a_record)
    session.commit()
    session.close()

    session = get_db_sessionmaker(a_sqlite_url)()
    test = session.query(PendingAnonRecord).first()
    assert not test.last_check
    assert not test.last_status

    test.last_check = last_check
    test.last_status = last_status

    session.add(test)
    session.commit()
    session.close()

    session = get_db_sessionmaker(a_sqlite_url)()
    test = session.query(PendingAnonRecord).first()
    assert test.study_folder == study_folder
    assert test.last_check == last_check


def test_idis_send_records(a_sqlite_url):
    """the records object makes interacting with db slightly cleaner """

    records = IDISSendRecords(session_maker=get_db_sessionmaker(a_sqlite_url))

    with records.get_session() as session:
        assert not session.get_all()
        session.add(study_folder=Path("test/something"), job_id=99, server_name="p03")

        session.add(study_folder=Path("test/something2"), job_id=100, server_name="p03")

    with records.get_session() as session:
        assert len(session.get_all()) == 2
        assert (
            session.get_for_study_folder(study_folder=Path("test/something2")).job_id
            == 100
        )
        assert not session.get_for_study_folder(study_folder=Path("test/something5"))

        session.delete(
            session.get_for_study_folder(study_folder=Path("test/something2"))
        )

    with records.get_session() as session:
        assert len(session.get_all()) == 1
        assert session.get_for_job_id(99)
        assert not session.get_for_job_id(100)


def test_object_field_persistence(a_sqlite_url):
    """Got into sqlalchemy object state trouble again.
    So when you create an ORM object, add it to as session, Expunge the object,
     close the session, why are all the object's fields inaccessible?

    """
    records = IDISSendRecords(session_maker=get_db_sessionmaker(a_sqlite_url))

    with records.get_session() as session:
        record = session.add(
            study_folder=Path("test"), job_id=1, server_name="test_server"
        )
    _ = record.server_name  # This should not raise an unbound exception

    # now can be change this unbound object and then commit the results?
    record.server_name = "changed"
    with records.get_session() as session:
        session.add_record(record)
    # yes



"""Object Relational Map for mapping idissend objects to database tables

"""
from pathlib import Path
from sqlalchemy import types
from sqlalchemy.ext.declarative.api import declarative_base
from sqlalchemy.sql.schema import Column
from sqlalchemy.sql.sqltypes import Integer, String, DateTime
from sqlalchemy.sql.type_api import TypeDecorator

Base = declarative_base()


class PathType(TypeDecorator):
    """A sqlalchemy type for pathlib.Path instances"""

    impl = types.String

    def process_bind_param(self, value, dialect):
        return str(value)

    def process_literal_param(self, value, dialect):
        return str(value)

    def process_result_value(self, value, dialect):
        return Path(value)


class PendingAnonRecord(Base):
    """Records for studies that er pending anonymization"""

    __tablename__ = "pending_anon"

    id = Column(Integer, primary_key=True)
    study_folder = Column(PathType)
    job_id = Column(Integer, unique=True)
    server_name = Column(String(length=256))
    last_status = Column(String(length=128))
    last_check = Column(DateTime)



"""Classes and functions to be able to hold on to records beyond python executions
"""
import sqlalchemy

from datetime import datetime
from idissend.exceptions import IDISSendException
from idissend.orm import Base, PendingAnonRecord
from pathlib import Path
from sqlalchemy.orm.session import sessionmaker
from sqlalchemy import create_engine
from typing import List, Optional

Session = sessionmaker()


class IDISSendRecordsSession:
    """An open session to a records database

    Removes some of the clutter of sqlalchemy backend. Properly typed method
    signatures and handling of session close with context manager

    Examples
    --------
    with IDISSendRecordsSession(sqlalchemy_session) as session:
        session.do_things()

    Notes
    -----
    Session is closed automatically after 'with' context is left. Objects returned
    from IDISSendRecordsSession methods can still be used after session close,
    but need to be added again to new session to persist any changes

    See Also
    --------
    You can use IDISSendRecords to generate IDISSendRecordsSession objects

    """

    def __init__(self, session: sqlalchemy.orm.session):
        self.session = session

    def __enter__(self):
        return self

    def __exit__(self, *_, **__):
        self.close()

    def close(self):
        """Close tries to keep as much information alive as possible in the objects
        returned by IDISSendRecordsSession methods.

        By default, sqlalchemy will invalidate any object fields after close.
        This can be stopped by expunge, but this only saved fields that have been
        accessed before. This is quite annoying here because sessions are quite
        short lived and often not all fields are accessed. Instead of hacking
        something to access all fields, the following amazing commands will do:
        """
        # flush all changes, obtain pk's etc, but do not commit
        self.session.flush()
        # detach all objects from session. Persists all object fields after close
        self.session.expunge_all()
        # Write to db. Without expunge all fields would now have become invalid
        self.session.commit()
        # be done with it
        self.session.close()

    def get_all(self) -> List[PendingAnonRecord]:
        return self.session.query(PendingAnonRecord).all()

    def get_for_study_folder(self, study_folder: Path) -> Optional[PendingAnonRecord]:
        """Get record for the given study folder. Returns None if not found"""
        return (
            self.session.query(PendingAnonRecord)
            .filter(PendingAnonRecord.study_folder == study_folder)
            .first()
        )

    def get_for_job_id(self, job_id: int) -> Optional[PendingAnonRecord]:
        """Get record for the given job_id. Returns None if not found"""
        return (
            self.session.query(PendingAnonRecord)
            .filter(PendingAnonRecord.job_id == job_id)
            .first()
        )

    def add(
        self,
        study_folder: Path,
        job_id: int,
        server_name: str,
        last_status: Optional[str] = None,
        last_check: Optional[datetime] = None,
    ) -> PendingAnonRecord:
        """Create a records with the given parameters.

        Made this instead of just using PendingAnonRecord to be explicit about
        argument types and default arguments
        """

        record = PendingAnonRecord(
            study_folder=study_folder,
            job_id=job_id,
            server_name=server_name,
            last_status=last_status,
            last_check=last_check,
        )
        self.add_record(record)
        return record

    def add_record(self, record: PendingAnonRecord):
        self.session.add(record)

    def delete(self, record: PendingAnonRecord):
        self.session.delete(record)


def get_db_sessionmaker(db_url) -> sqlalchemy.orm.session.sessionmaker:
    """Returns a session on a anonqa sqlite database in the given file.
    Creates db if it does not exist

    Parameters
    ----------
    db_url: String
        Sqlalchemy database url.
        See https://docs.sqlalchemy.org/en/13/core/engines.html#database-urls


    Returns
    -------
    sqlalchemy.orm.session.Session
        A session on the database in db_filename
    """
    engine = create_engine(db_url, echo=False)
    Base.metadata.create_all(engine, checkfirst=True)  # Create if needed
    Session.configure(bind=engine)
    return Session


def get_memory_only_sessionmaker() -> sqlalchemy.orm.session.sessionmaker:
    """Session on db that exists only in memory. Will stop existing when closed"""
    engine = create_engine("sqlite://", echo=False)
    Base.metadata.create_all(engine, checkfirst=True)  # Create if needed
    Session.configure(bind=engine)
    return Session


class IDISSendRecords:
    """A thing that holds persistent records for idissend

    This is the object that gets passed around in idisssend. When actual db
    transactions are needed, use get_session and the 'with' statement:

    records = IDISSendRecords(session_maker)

    with records.get_session() as session:
        session.do_things()

    """

    def __init__(self, session_maker: sqlalchemy.orm.session.sessionmaker):
        self.session_maker = session_maker

    def get_session(self) -> IDISSendRecordsSession:
        return IDISSendRecordsSession(session=self.session_maker())


class DBLoadException(IDISSendException):
    pass



import pytest

from idissend.core import Study, StudyPushException, PushStudyCallbackException
from itertools import cycle
from pathlib import Path
from tests.factories import StudyFactory
from unittest.mock import Mock


def test_incoming_folder(an_incoming_stage):
    """Basic functions for an incoming folder"""
    folder = an_incoming_stage
    studies = folder.get_all_studies()

    assert len(studies) == 3
    assert "project1" in [x.stream.name for x in studies]
    assert "project2" in [x.stream.name for x in studies]


def test_cooldown(monkeypatch):
    """Studies are considered complete after a cool_down period. Does this work?"""
    # a study with some files
    some_files = [Path(), Path(), Path()]
    study: Study = StudyFactory()
    study.get_files = lambda: some_files  # don't check path on disk, just mock

    # checking the age of these files will yield 10, 11, 12, 10, etc..
    monkeypatch.setattr(
        "idissend.core.IncomingFile.age", Mock(side_effect=cycle([10, 11, 12]))
    )

    assert study.is_older_than(5)
    assert not study.is_older_than(11)
    assert not study.is_older_than(15)


def test_push_study(an_incoming_stage, some_stages):
    """Basic transfer of studies between stages
    """
    incoming = an_incoming_stage
    stage1 = some_stages[0]

    assert len(incoming.get_all_studies()) == 3
    assert len(stage1.get_all_studies()) == 0

    stage1.push_study(incoming.get_all_studies()[0])
    assert len(incoming.get_all_studies()) == 2
    assert len(stage1.get_all_studies()) == 1

    stage1.push_study(incoming.get_all_studies()[0])
    assert len(incoming.get_all_studies()) == 1
    assert len(stage1.get_all_studies()) == 2


def test_push_study_exception_missing_stream(a_stage, a_study):
    """Stream does not exist in target stage
    """
    a_stage.streams.remove(a_study.stream)
    with pytest.raises(StudyPushException):
        a_stage.push_study(a_study)


def test_push_study_exceptions(a_stage, a_study):
    """ Data does not exist for study (unexpected but not impossible)
    """
    # remove data for study
    a_study.get_path().rename(a_study.get_path().parent / "removed")
    with pytest.raises(StudyPushException):
        a_stage.push_study(a_study)


def test_push_study_to_itself(a_study):
    """ Cannot push to a stage a study is in already
    """
    with pytest.raises(StudyPushException):
        a_study.stage.push_study(a_study)


def test_push_study_callback_fail(a_stage, a_study):
    """push callback fails for some reason on target stage
    """

    assert len(a_study.stage.get_all_studies()) == 3
    assert len(a_stage.get_all_studies()) == 0

    a_stage.push_study_callback = Mock(
        side_effect=PushStudyCallbackException("Something really went wrong here")
    )

    with pytest.raises(StudyPushException):
        a_stage.push_study(a_study)

    # study move should have been rolled back
    assert len(a_study.stage.get_all_studies()) == 3
    assert len(a_stage.get_all_studies()) == 0


def test_push_study_out_of_space(a_stage, a_study, monkeypatch):
    """out of space on target stage
    """
    monkeypatch.setattr(
        "idissend.core.shutil.move", Mock(side_effect=IOError("out of space"))
    )

    with pytest.raises(StudyPushException):
        a_stage.push_study(a_study)



import logging
from pathlib import Path
from unittest.mock import Mock

import pytest
from anonapi.client import ClientToolException
from anonapi.paths import UNCPath, UNCMapping, UNCMap
from anonapi.responses import JobsInfoList
from anonapi.testresources import (
    RemoteAnonServerFactory,
    JobInfoFactory,
    JobStatus,
    MockAnonClientTool,
)

from idissend.core import StudyPushException
from idissend.persistence import IDISSendRecords, get_memory_only_sessionmaker
from idissend.stages import (
    PendingAnon,
    IDISConnection,
    UnknownServerException,
    IDISCommunicationException,
    RecordNotFoundException,
    Trash,
)
from tests.factories import StreamFactory


@pytest.fixture
def a_pending_anon_stage_with_data(an_empty_pending_stage, some_studies) -> PendingAnon:
    """A pending stage to which three studies have been pushed"""
    for study in some_studies:
        an_empty_pending_stage.push_study(study)
    return an_empty_pending_stage


@pytest.fixture
def a_trash_stage(a_pending_anon_stage_with_data, tmpdir) -> Trash:
    """A trash stage with temp path on disk and the same streams as
    a_pending_anon_stage_with_data"""

    trash = Trash(
        name="Trash",
        streams=a_pending_anon_stage_with_data.streams,
        path=Path(tmpdir) / "trash",
    )
    trash.assert_all_paths()
    return trash


def test_idis_connection(an_idis_connection):
    an_idis_connection.servers = [
        RemoteAnonServerFactory(name="server1"),
        RemoteAnonServerFactory(name="server2"),
    ]
    assert an_idis_connection.get_server("server1")
    with pytest.raises(UnknownServerException):
        an_idis_connection.get_server("unknown server")


def test_pending_anon_push(an_empty_pending_stage, mock_anon_client_tool, a_study):
    """Pending should create IDIS jobs when studies are pushed to it
     """
    # make sure initial state is as expected:
    assert len(mock_anon_client_tool.mock_calls) == 0  # No calls to IDIS
    with an_empty_pending_stage.records.get_session() as session:
        assert len(session.get_all()) == 0  # No records

        # push some data to stage
        an_empty_pending_stage.push_study(a_study)

        # A job should have been made with IDIS
        assert mock_anon_client_tool.create_path_job.called

        # A record of this should have been made
        assert len(session.get_all()) == 1


def test_pending_anon_push_unc_paths(
    an_empty_pending_stage, mock_anon_client_tool, a_study
):
    """ Created jobs should have UNC input and output. Otherwise IDIS will just
    choke on them (design flaw definitely)"""

    # study with local source and destination. As is these paths make no sense
    # on an IDIS server

    a_stream = StreamFactory(output_folder=Path("/mnt/datashare/some/folder"))
    an_empty_pending_stage.streams.append(a_stream)

    # map local paths to UNC paths to make translation possible
    mapping = UNCMapping(
        maps=[
            UNCMap(local=Path("/"), unc=UNCPath(r"\\server\share")),
            UNCMap(
                local=Path("/mnt/datashare"), unc=UNCPath(r"\\dataserver\datashare")
            ),
        ]
    )
    an_empty_pending_stage.unc_mapping = mapping

    new_study = an_empty_pending_stage.push_study(a_study, a_stream)

    # A job should have been made with IDIS
    assert mock_anon_client_tool.create_path_job.called

    # and the paths should be UNC
    idis_source_path = mock_anon_client_tool.create_path_job.call_args[1]["source_path"]
    idis_destination_path = mock_anon_client_tool.create_path_job.call_args[1][
        "destination_path"
    ]

    assert UNCPath.is_unc(idis_source_path)
    assert UNCPath.is_unc(idis_destination_path)

    # If a path is set that cannot be translated the push should fail
    a_stream = StreamFactory(output_folder=Path(r"C:\data"))
    an_empty_pending_stage.streams.append(a_stream)
    with pytest.raises(StudyPushException) as e:
        an_empty_pending_stage.push_study(new_study, a_stream)
    assert "could not be mapped" in str(e)


def test_pending_anon_push_idis_exception(
    an_empty_pending_stage, mock_anon_client_tool, a_study
):
    """Pending should create IDIS jobs. What happens when these fail?
     """
    # contact IDIS will not work
    mock_anon_client_tool.create_path_job = Mock(
        side_effect=ClientToolException("Terrible API error")
    )

    # pushing should raise
    with pytest.raises(StudyPushException):
        an_empty_pending_stage.push_study(a_study)

    # any copy or move should have been rolled back
    assert len(an_empty_pending_stage.get_all_studies()) == 0


def test_pending_anon_check_status(
    mock_anon_client_tool, a_pending_anon_stage_with_data
):
    """A pending stage should be able to check on job status with IDIS
    """

    stage = a_pending_anon_stage_with_data

    with stage.records.get_session() as session:
        # make sure two different servers were used for creating jobs
        session.get_all()[0].server_name = stage.idis_connection.servers[0].name
        session.get_all()[1].server_name = stage.idis_connection.servers[1].name

    # get record for each stage
    studies = stage.get_all_studies()

    # contact IDIS to get the latest on the jobs corresponding to each study
    studies = stage.update_records(studies)

    # get important groups: Studies finished, errored, still pending
    finished = [x for x in studies if x.last_status == JobStatus.DONE]
    errored = [x for x in studies if x.last_status == JobStatus.ERROR]
    still_going = [x for x in studies if x.last_status == JobStatus.ACTIVE]
    cancelled = [x for x in studies if x.last_status == JobStatus.INACTIVE]

    assert len(finished) == 1
    assert len(errored) == 1
    assert len(cancelled) == 1
    assert len(still_going) == 0


def test_pending_anon_check_status_exceptions(
    mock_anon_client_tool, a_pending_anon_stage_with_data
):
    """Handle errors in interaction with IDIS gracefully

    """

    stage = a_pending_anon_stage_with_data
    studies = stage.get_all_studies()

    # Contacting IDIS will not work at all (for example when server is down)
    mock_anon_client_tool.get_job_info_list = Mock(
        side_effect=ClientToolException("Terrible API error")
    )

    with pytest.raises(IDISCommunicationException):
        stage.update_records(studies)

    # Contacting IDIS will work, but not all job ids are found (only id=1 is returned)
    mock_anon_client_tool.get_job_info_list = Mock(
        return_value=JobsInfoList(job_infos=[JobInfoFactory(job_id=1)])
    )

    with pytest.raises(IDISCommunicationException):
        stage.update_records(studies)


def test_pending_anon_missing_record(
    a_pending_anon_stage_with_data, a_trash_stage, mock_anon_client_tool, a_records_db
):
    """Pending stage should be able to handle missing records
     """
    # This stage has 3 studies already pushed to it. Verify studies,
    # IDIS jobs and records
    pending = a_pending_anon_stage_with_data
    assert len(pending.get_all_studies()) == 3
    with a_records_db.get_session() as session:
        assert len(session.get_all()) == 3

    # now something bad happens for some reason. One record is lost!
    with a_records_db.get_session() as session:
        session.delete(session.get_all()[0])

    # regular get_all_studies will fail because missing record
    with pytest.raises(RecordNotFoundException):
        pending.get_all_studies()

    # however you can obtain the studies that cause the exception and remove them
    orphaned = pending.get_all_orphaned_studies()
    a_trash_stage.push_studies(orphaned)

    # now the remaining studies can be obtained again
    assert len(pending.get_all_studies()) == 2


def test_trash_stage(a_pending_anon_stage_with_data, a_trash_stage, caplog):
    """Emptying trash should work and be logged"""
    caplog.set_level(logging.DEBUG)

    a_trash_stage.push_studies(a_pending_anon_stage_with_data.get_all_studies())
    assert len(a_trash_stage.get_all_studies()) == 3

    a_trash_stage.delete_all()
    assert len(a_trash_stage.get_all_studies()) == 0
    assert "Removing data for 3 studies" in caplog.text



"""
Integration test of a pipeline consisting of different streams and stages.
For testing log messages etc.
"""
import logging
from pathlib import Path
from unittest.mock import Mock

import pytest
from anonapi.client import ClientToolException

from idissend.core import Stage
from idissend.exceptions import IDISSendException
from idissend.pipeline import DefaultPipeline
from idissend.stages import (
    Trash,
    IDISCommunicationException,
    RecordNotFoundException,
    CoolDown,
)


@pytest.fixture
def a_pipeline(
    an_incoming_stage, an_empty_pending_stage, an_idis_connection, tmp_path, caplog
):
    """A default pipeline with all-mocked connections to outside servers.
    Integration test light. Useful for checking log messages etc."""
    # capture all logs
    caplog.set_level(logging.DEBUG)

    # make sure all stages have the same streams
    streams = an_incoming_stage.streams
    an_empty_pending_stage.streams = streams
    finished = CoolDown(
        name="finished", path=Path(tmp_path) / "finished", streams=streams, cool_down=0
    )
    trash = Trash(name="Trash", path=Path(tmp_path) / "trash", streams=streams)
    errored = Stage(name="errored", path=Path(tmp_path) / "errored", streams=streams)

    return DefaultPipeline(
        incoming=an_incoming_stage,
        pending=an_empty_pending_stage,
        finished=finished,
        trash=trash,
        errored=errored,
    )


def test_pipline_regular_operation(a_pipeline, caplog):
    """ check logs for regular operation """
    a_pipeline.run_once()
    a_pipeline.run_once()

    # this is just meant for visual inspection to develop logs. No real tests here
    assert "Running once" in caplog.text


def test_pipeline_idis_exceptions(a_pipeline, caplog, an_idis_connection):
    """What happens when IDIS connection fails"""

    an_idis_connection.client_tool.get_job_info_list = Mock(
        side_effect=ClientToolException(
            "IDIS fell over. Out of the window. Into a pond. "
            "Full of sharks. Radioactive Sharks. Connection lost"
        )
    )
    a_pipeline.run_once()  # import from incoming to pending
    status_before = a_pipeline.get_status()
    with pytest.raises(IDISSendException) as e:
        a_pipeline.run_once()  # IDIS call from pending will raise exception

    assert e.type == IDISCommunicationException
    assert status_before == a_pipeline.get_status()  # nothing should have changed


def test_pipeline_record_not_found_exception(a_pipeline, caplog, a_records_db):
    """What happens when a single record is not found. This can occur after errors,
    or data for studies is somehow moved into the streams outside idissend"""

    # push to pending and create IDIS jobs
    a_pipeline.run_once()  # import from incoming to pending

    # now remove a record
    with a_records_db.get_session() as session:
        session.delete(session.get_all()[0])

    # running again will fail in the pending stage because one study now has no
    # record, so it is unknown which IDIS job has been made for it
    with pytest.raises(RecordNotFoundException):
        a_pipeline.run_once()  # import from incoming to pending



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0', 'sqlalchemy', 'anonapi>=1.0.10']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

setup(
    author="Sjoerd Kerkstra",
    author_email='w.s.kerkstra@gmail.com',
    python_requires='~=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Automate sending incoming files to IDIS anonymization server",
    entry_points={
        'console_scripts': [
            'idissend=idissend.cli:main',
        ],
    },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='idissend',
    name='idissend',
    packages=find_packages(include=['idissend', 'idissend.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/sjoerdk/idissend',
    version='2.0.3',
    zip_safe=False,
)
